@@ -1,6 +1,6 @@
 /*
  * sar, sadc, sadf, mpstat and iostat common routines.
- * (C) 1999-2023 by Sebastien GODARD (sysstat <at> orange.fr)
+ * (C) 1999-2025 by Sebastien GODARD (sysstat <at> orange.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -77,13 +77,29 @@ char persistent_name_type[MAX_FILE_LEN];
 
 /*
  ***************************************************************************
- * Print sysstat version number and exit.
+ * Print sysstat version number, environment variables and exit.
+ *
+ * IN:
+ * @env		Array with environment variable names.
+ * @n		Number of environment variables.
  ***************************************************************************
  */
-void print_version(void)
+void print_version(char *env[], int n)
 {
+	char *e;
+	int i;
+
+	/* Display contents of environment variables */
+	for (i = 0; i < n; i++) {
+		if ((e = __getenv(env[i])) != NULL) {
+			printf("%s=%s\n", env[i], e);
+		}
+	}
+
+	/* Print sysstat version number */
 	printf(_("sysstat version %s\n"), VERSION);
 	printf("(C) Sebastien Godard (sysstat <at> orange.fr)\n");
+
 	exit(0);
 }
 
@@ -138,9 +154,10 @@ time_t get_xtime(struct tm *rectime, int d_off, int utc)
 time_t get_time(struct tm *rectime, int d_off)
 {
 	static int utc = 0;
-	char *e;
 
 	if (!utc) {
+		char *e;
+
 		/* Read environment variable value once */
 		if ((e = __getenv(ENV_TIME_DEFTM)) != NULL) {
 			utc = !strcmp(e, K_UTC);
@@ -425,20 +442,56 @@ int check_dir(char *dirname)
  * @val1	First value.
  * @val2	Second value.
  * @val3	Third value.
+ *
+ * RETURNS:
+ * Multiplication of the 3 values.
  ***************************************************************************
  */
-void check_overflow(unsigned int val1, unsigned int val2,
-		    unsigned int val3)
+size_t mul_check_overflow3(size_t val1, size_t val2, size_t val3)
 {
-	if ((unsigned long long) val1 * (unsigned long long) val2 *
-	    (unsigned long long) val3 > UINT_MAX) {
+	if ((val1 != 0) && (val2 != 0) && (val3 != 0) &&
+	    ((UINT_MAX < val1) ||
+	     (UINT_MAX / val1 < val2) ||
+	     (UINT_MAX / (val1 * val2) < val3))) {
 #ifdef DEBUG
-		fprintf(stderr, "%s: Overflow detected (%llu). Aborting...\n",
-			__FUNCTION__, (unsigned long long) val1 * (unsigned long long) val2 *
-			(unsigned long long) val3);
+		fprintf(stderr, "%s: Overflow detected (%zu,%zu,%zu). Aborting...\n",
+			__FUNCTION__, val1, val2, val3);
 #endif
-	exit(4);
-		}
+		exit(4);
+	}
+
+	return (val1 * val2 * val3);
+}
+
+/*
+ * **************************************************************************
+ * Check if the multiplication of the 4 values may be greater than UINT_MAX.
+ *
+ * IN:
+ * @val1	First value.
+ * @val2	Second value.
+ * @val3	Third value.
+ * @val4	Fourth value.
+ *
+ * RETURNS:
+ * Multiplication of the 4 values.
+ ***************************************************************************
+ */
+size_t mul_check_overflow4(size_t val1, size_t val2, size_t val3, size_t val4)
+{
+	if ((val1 != 0) && (val2 != 0) && (val3 != 0) && (val4 != 0) &&
+	    ((UINT_MAX < val1) ||
+	     (UINT_MAX / val1 < val2) ||
+	     (UINT_MAX / (val1 * val2) < val3) ||
+	     (UINT_MAX / (val1 * val2 * val3) < val4))) {
+#ifdef DEBUG
+		fprintf(stderr, "%s: Overflow detected (%zu,%zu,%zu,%zu). Aborting...\n",
+			__FUNCTION__, val1, val2, val3, val4);
+#endif
+		exit(4);
+	}
+
+	return (val1 * val2 * val3 * val4);
 }
 
 #ifndef SOURCE_SADC
@@ -488,9 +541,10 @@ unsigned int get_devmap_major(void)
 int is_iso_time_fmt(void)
 {
 	static int is_iso = -1;
-	char *e;
 
 	if (is_iso < 0) {
+		char *e;
+
 		is_iso = (((e = __getenv(ENV_TIME_FMT)) != NULL) && !strcmp(e, K_ISO));
 	}
 	return is_iso;
@@ -726,6 +780,41 @@ char *device_name(char *name)
 	return out;
 }
 
+/*
+ ***************************************************************************
+ * Escape a '\' character in a JSON string (replace '\' with "\\").
+ *
+ * IN:
+ * @str		String which may contain '\' characters.
+ *
+ * RETURNS:
+ * String where '\' characters have been escaped.
+ ***************************************************************************
+ */
+char *escape_bs_char(char *str)
+{
+	static char buffer[MAX_NAME_LEN];
+	int i = 0, j = 0;
+
+	while (str[i] != '\0' && j < MAX_NAME_LEN - 1) {
+		if (str[i] == '\\') {
+			if (j < MAX_NAME_LEN - 2) {
+				buffer[j++] = '\\';
+				buffer[j++] = '\\';
+			} else {
+				break;
+			}
+		} else {
+			buffer[j++] = str[i];
+		}
+		i++;
+	}
+
+	buffer[j] = '\0';
+
+	return buffer;
+}
+
 /*
  ***************************************************************************
  * Workaround for CPU counters read from /proc/stat: Dyn-tick kernels
@@ -999,7 +1088,7 @@ char *get_persistent_name_from_pretty(char *pretty)
 	}
 	free (persist_names);
 
-	if (strlen(persist_name) <= 0)
+	if (!strlen(persist_name))
 		return (NULL);
 
 	return persist_name;
@@ -1134,7 +1223,6 @@ char *get_device_name(unsigned int major, unsigned int minor, unsigned long long
 	static unsigned int dm_major = 0;
 	char *dev_name = NULL, *persist_dev_name = NULL, *bang;
 	static char sid[64], dname[MAX_NAME_LEN];
-	char xsid[32] = "", pn[16] = "";
 
 	if (disp_persist_name) {
 		persist_dev_name = get_persistent_name_from_pretty(get_devname(major, minor));
@@ -1145,11 +1233,13 @@ char *get_device_name(unsigned int major, unsigned int minor, unsigned long long
 	}
 	else {
 		if (use_stable_id && (wwn[0] != 0)) {
+			char xsid[32] = "", pn[16] = "";
+
 			if (wwn[1] != 0) {
 				sprintf(xsid, "%016llx", wwn[1]);
 			}
 			if (part_nr) {
-				sprintf(pn, "-%d", part_nr);
+				sprintf(pn, "-%u", part_nr);
 			}
 			snprintf(sid, sizeof(sid), "%#016llx%s%s", wwn[0], xsid, pn);
 			dev_name = sid;
@@ -1734,7 +1824,7 @@ int parse_values(char *strargv, unsigned char bitmap[], int max_val, const char
 				return 1;
 
 			for (i = val_low; i <= val; i++) {
-				bitmap[(i + 1) >> 3] |= 1 << ((i + 1) & 0x07);
+				SET_CPU_BITMAP(bitmap, i + 1);
 			}
 		}
 	}
@@ -1742,4 +1832,42 @@ int parse_values(char *strargv, unsigned char bitmap[], int max_val, const char
 	return 0;
 }
 
+/*
+ * **************************************************************************
+ * Write current sample's timestamp, either in plain or JSON format.
+ *
+ * IN:
+ * @tab		Number of tabs to print.
+ * @rectime	Current date and time.
+ * @xflags	Flag for common options and system state.
+ ***************************************************************************
+ */
+void write_sample_timestamp(int tab, struct tm *rectime, uint64_t xflags)
+{
+	char timestamp[TIMESTAMP_LEN];
+
+	if (DISPLAY_SEC_EPOCH(xflags)) {
+		snprintf(timestamp, sizeof(timestamp), "%ld", mktime(rectime));
+		timestamp[sizeof(timestamp) - 1] = '\0';
+	}
+	else if (DISPLAY_ISO(xflags)) {
+		strftime(timestamp, sizeof(timestamp), "%FT%T%z", rectime);
+	}
+	else {
+		strftime(timestamp, sizeof(timestamp), "%x %X", rectime);
+	}
+	if (DISPLAY_JSON_OUTPUT(xflags)) {
+		xprintf(tab, "\"timestamp\": \"%s\",", timestamp);
+	}
+	else {
+		printf("%s\n", timestamp);
+	}
+
+#ifdef DEBUG
+	if (DISPLAY_DEBUG(xflags)) {
+		fprintf(stderr, "%s\n", timestamp);
+	}
+#endif
+}
+
 #endif /* SOURCE_SADC undefined */
