@@ -1,6 +1,6 @@
 /*
  * pcp_stats.c: Functions used by sadf to create PCP archive files.
- * (C) 2019-2023 by Sebastien GODARD (sysstat <at> orange.fr)
+ * (C) 2019-2025 by Sebastien GODARD (sysstat <at> orange.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -20,7 +20,6 @@
  */
 
 #include "sa.h"
-#include "pcp_stats.h"
 
 #ifdef USE_NLS
 #include <locale.h>
@@ -79,8 +78,8 @@ __print_funct_t pcp_print_cpu_stats(struct activity *a, int curr)
 	for (i = 0; (i < a->nr_ini) && (i < a->bitmap->b_size + 1); i++) {
 
 		/* Should current CPU (including CPU "all") be displayed? */
-		if (!(a->bitmap->b_array[i >> 3] & (1 << (i & 0x07))) ||
-		    offline_cpu_bitmap[i >> 3] & (1 << (i & 0x07)))
+		if (!IS_CPU_SELECTED(a->bitmap->b_array, i) ||
+		    IS_CPU_OFFLINE(offline_cpu_bitmap, i))
 			/* Don't display CPU */
 			continue;
 
@@ -232,7 +231,7 @@ __print_funct_t pcp_print_irq_stats(struct activity *a, int curr)
 										  + i * a->msize);
 
 			/* Should current CPU (including CPU "all") be processed? */
-			if (masked_cpu_bitmap[c >> 3] & (1 << (c & 0x07)))
+			if (IS_CPU_SET(masked_cpu_bitmap, c))
 				/* No */
 				continue;
 
@@ -327,6 +326,12 @@ __print_funct_t pcp_print_paging_stats(struct activity *a, int curr)
 
 	snprintf(buf, sizeof(buf), "%llu", (unsigned long long) spc->pgsteal);
 	pmiPutValue("mem.vmstat.pgsteal_total", NULL, buf);
+
+	snprintf(buf, sizeof(buf), "%llu", (unsigned long long) spc->pgpromote);
+	pmiPutValue("mem.vmstat.pgpromote_success", NULL, buf);
+
+	snprintf(buf, sizeof(buf), "%llu", (unsigned long long) spc->pgdemote);
+	pmiPutValue("mem.vmstat.pgdemote_total", NULL, buf);
 #endif	/* HAVE_PCP */
 }
 
@@ -369,6 +374,99 @@ __print_funct_t pcp_print_io_stats(struct activity *a, int curr)
 #endif	/* HAVE_PCP */
 }
 
+/*
+ * **************************************************************************
+ * Display RAM memory utilization in PCP format.
+ *
+ * IN:
+ * @smc		Structure with statistics.
+ * @dispall	TRUE if all memory fields should be displayed.
+ ***************************************************************************
+ */
+void pcp_print_ram_memory_stats(struct stats_memory *smc, int dispall)
+{
+#ifdef HAVE_PCP
+	char buf[64];
+
+	snprintf(buf, sizeof(buf), "%lu", (unsigned long) (smc->tlmkb >> 10));
+	pmiPutValue("hinv.physmem", NULL, buf);
+
+	snprintf(buf, sizeof(buf), "%llu", smc->tlmkb);
+	pmiPutValue("mem.physmem", NULL, buf);
+
+	snprintf(buf, sizeof(buf), "%llu", smc->frmkb);
+	pmiPutValue("mem.util.free", NULL, buf);
+
+	snprintf(buf, sizeof(buf), "%llu", smc->availablekb);
+	pmiPutValue("mem.util.available", NULL, buf);
+
+	snprintf(buf, sizeof(buf), "%llu", smc->tlmkb - smc->availablekb);
+	pmiPutValue("mem.util.used", NULL, buf);
+
+	snprintf(buf, sizeof(buf), "%llu", smc->bufkb);
+	pmiPutValue("mem.util.bufmem", NULL, buf);
+
+	snprintf(buf, sizeof(buf), "%llu", smc->camkb);
+	pmiPutValue("mem.util.cached", NULL, buf);
+
+	snprintf(buf, sizeof(buf), "%llu", smc->comkb);
+	pmiPutValue("mem.util.committed_AS", NULL, buf);
+
+	snprintf(buf, sizeof(buf), "%llu", smc->activekb);
+	pmiPutValue("mem.util.active", NULL, buf);
+
+	snprintf(buf, sizeof(buf), "%llu", smc->inactkb);
+	pmiPutValue("mem.util.inactive", NULL, buf);
+
+	snprintf(buf, sizeof(buf), "%llu", smc->dirtykb);
+	pmiPutValue("mem.util.dirty", NULL, buf);
+
+	snprintf(buf, sizeof(buf), "%llu", smc->shmemkb);
+	pmiPutValue("mem.util.shared", NULL, buf);
+
+	if (dispall) {
+		snprintf(buf, sizeof(buf), "%llu", smc->anonpgkb);
+		pmiPutValue("mem.util.anonpages", NULL, buf);
+
+		snprintf(buf, sizeof(buf), "%llu", smc->slabkb);
+		pmiPutValue("mem.util.slab", NULL, buf);
+
+		snprintf(buf, sizeof(buf), "%llu", smc->kstackkb);
+		pmiPutValue("mem.util.kernelStack", NULL, buf);
+
+		snprintf(buf, sizeof(buf), "%llu", smc->pgtblkb);
+		pmiPutValue("mem.util.pageTables", NULL, buf);
+
+		snprintf(buf, sizeof(buf), "%llu", smc->vmusedkb);
+		pmiPutValue("mem.util.vmallocUsed", NULL, buf);
+	}
+#endif	/* HAVE_PCP */
+}
+
+/*
+ * **************************************************************************
+ * Display swap memory utilization in PCP format.
+ *
+ * IN:
+ * @smc		Structure with statistics.
+ ***************************************************************************
+ */
+void pcp_print_swap_memory_stats(struct stats_memory *smc)
+{
+#ifdef HAVE_PCP
+	char buf[64];
+
+	snprintf(buf, sizeof(buf), "%llu", smc->frskb);
+	pmiPutValue("mem.util.swapFree", NULL, buf);
+
+	snprintf(buf, sizeof(buf), "%llu", smc->tlskb);
+	pmiPutValue("mem.util.swapTotal", NULL, buf);
+
+	snprintf(buf, sizeof(buf), "%llu", smc->caskb);
+	pmiPutValue("mem.util.swapCached", NULL, buf);
+#endif	/* HAVE_PCP */
+}
+
 /*
  ***************************************************************************
  * Display memory statistics in PCP format.
@@ -381,74 +479,15 @@ __print_funct_t pcp_print_io_stats(struct activity *a, int curr)
 __print_funct_t pcp_print_memory_stats(struct activity *a, int curr)
 {
 #ifdef HAVE_PCP
-	char buf[64];
 	struct stats_memory
 		*smc = (struct stats_memory *) a->buf[curr];
 
 	if (DISPLAY_MEMORY(a->opt_flags)) {
-
-		snprintf(buf, sizeof(buf), "%lu", (unsigned long) (smc->tlmkb >> 10));
-		pmiPutValue("hinv.physmem", NULL, buf);
-
-		snprintf(buf, sizeof(buf), "%llu", smc->tlmkb);
-		pmiPutValue("mem.physmem", NULL, buf);
-
-		snprintf(buf, sizeof(buf), "%llu", smc->frmkb);
-		pmiPutValue("mem.util.free", NULL, buf);
-
-		snprintf(buf, sizeof(buf), "%llu", smc->availablekb);
-		pmiPutValue("mem.util.available", NULL, buf);
-
-		snprintf(buf, sizeof(buf), "%llu", smc->tlmkb - smc->frmkb);
-		pmiPutValue("mem.util.used", NULL, buf);
-
-		snprintf(buf, sizeof(buf), "%llu", smc->bufkb);
-		pmiPutValue("mem.util.bufmem", NULL, buf);
-
-		snprintf(buf, sizeof(buf), "%llu", smc->camkb);
-		pmiPutValue("mem.util.cached", NULL, buf);
-
-		snprintf(buf, sizeof(buf), "%llu", smc->comkb);
-		pmiPutValue("mem.util.committed_AS", NULL, buf);
-
-		snprintf(buf, sizeof(buf), "%llu", smc->activekb);
-		pmiPutValue("mem.util.active", NULL, buf);
-
-		snprintf(buf, sizeof(buf), "%llu", smc->inactkb);
-		pmiPutValue("mem.util.inactive", NULL, buf);
-
-		snprintf(buf, sizeof(buf), "%llu", smc->dirtykb);
-		pmiPutValue("mem.util.dirty", NULL, buf);
-
-		if (DISPLAY_MEM_ALL(a->opt_flags)) {
-
-			snprintf(buf, sizeof(buf), "%llu", smc->anonpgkb);
-			pmiPutValue("mem.util.anonpages", NULL, buf);
-
-			snprintf(buf, sizeof(buf), "%llu", smc->slabkb);
-			pmiPutValue("mem.util.slab", NULL, buf);
-
-			snprintf(buf, sizeof(buf), "%llu", smc->kstackkb);
-			pmiPutValue("mem.util.kernelStack", NULL, buf);
-
-			snprintf(buf, sizeof(buf), "%llu", smc->pgtblkb);
-			pmiPutValue("mem.util.pageTables", NULL, buf);
-
-			snprintf(buf, sizeof(buf), "%llu", smc->vmusedkb);
-			pmiPutValue("mem.util.vmallocUsed", NULL, buf);
-		}
+		pcp_print_ram_memory_stats(smc, DISPLAY_MEM_ALL(a->opt_flags));
 	}
 
 	if (DISPLAY_SWAP(a->opt_flags)) {
-
-		snprintf(buf, sizeof(buf), "%llu", smc->frskb);
-		pmiPutValue("mem.util.swapFree", NULL, buf);
-
-		snprintf(buf, sizeof(buf), "%llu", smc->tlskb);
-		pmiPutValue("mem.util.swapTotal", NULL, buf);
-
-		snprintf(buf, sizeof(buf), "%llu", smc->caskb);
-		pmiPutValue("mem.util.swapCached", NULL, buf);
+		pcp_print_swap_memory_stats(smc);
 	}
 #endif	/* HAVE_PCP */
 }
@@ -1465,7 +1504,7 @@ __print_funct_t pcp_print_pwr_cpufreq_stats(struct activity *a, int curr)
 		spc = (struct stats_pwr_cpufreq *) ((char *) a->buf[curr] + i * a->msize);
 
 		/* Should current CPU (including CPU "all") be displayed? */
-		if (!(a->bitmap->b_array[i >> 3] & (1 << (i & 0x07))))
+		if (!IS_CPU_SELECTED(a->bitmap->b_array, i))
 			/* No */
 			continue;
 
@@ -1684,7 +1723,7 @@ __print_funct_t pcp_print_pwr_usb_stats(struct activity *a, int curr)
 		suc = (struct stats_pwr_usb *) ((char *) a->buf[curr] + i * a->msize);
 		sprintf(instance, "usb%d", i);
 
-		snprintf(buf, sizeof(buf), "%d", suc->bus_nr);
+		snprintf(buf, sizeof(buf), "%u", suc->bus_nr);
 		pmiPutValue("power.usb.bus", instance, buf);
 
 		snprintf(buf, sizeof(buf), "%x", suc->vendor_id);
@@ -1796,8 +1835,8 @@ __print_funct_t pcp_print_softnet_stats(struct activity *a, int curr)
 	for (i = 0; (i < a->nr_ini) && (i < a->bitmap->b_size + 1); i++) {
 
 		/* Should current CPU (including CPU "all") be displayed? */
-		if (!(a->bitmap->b_array[i >> 3] & (1 << (i & 0x07))) ||
-		    offline_cpu_bitmap[i >> 3] & (1 << (i & 0x07)))
+		if (!IS_CPU_SELECTED(a->bitmap->b_array, i) ||
+		    IS_CPU_OFFLINE(offline_cpu_bitmap, i))
 			/* No */
 			continue;
 
@@ -1934,7 +1973,7 @@ __print_funct_t pcp_print_psiio_stats(struct activity *a, int curr)
 	snprintf(buf, sizeof(buf), "%f", (double) psic->full_aio_300 / 100);
 	pmiPutValue("kernel.all.pressure.io.full.avg", "5 minute", buf);
 
-	snprintf(buf, sizeof(buf), "%f", (double) psic->full_io_total);
+	snprintf(buf, sizeof(buf), "%llu", psic->full_io_total);
 	pmiPutValue("kernel.all.pressure.io.full.total", NULL, buf);
 #endif	/* HAVE_PCP */
 }
@@ -1976,7 +2015,7 @@ __print_funct_t pcp_print_psimem_stats(struct activity *a, int curr)
 	snprintf(buf, sizeof(buf), "%f", (double) psic->full_amem_300 / 100);
 	pmiPutValue("kernel.all.pressure.memory.full.avg", "5 minute", buf);
 
-	snprintf(buf, sizeof(buf), "%f", (double) psic->full_mem_total);
+	snprintf(buf, sizeof(buf), "%llu", psic->full_mem_total);
 	pmiPutValue("kernel.all.pressure.memory.full.total", NULL, buf);
 #endif	/* HAVE_PCP */
 }
